use anyhow::Result;
use dns_lookup::lookup_host;
use mountinfo::{MountInfo, ReadWrite, MountPoint};
use ping::{dgramsock, rawsock::ping};
use std::{ffi::OsStr, fs, path::PathBuf, sync::mpsc::{self, Receiver, Sender}, thread, time::Duration};
use sys_mount::{unmount, UnmountFlags};

fn main() {
    let mtab = MountInfo::new().unwrap();
    let mounting_points = mtab.mounting_points;
    let relevant_mounts = mounting_points.iter().filter(|&mount_point| {
        match &mount_point.fstype {
            mountinfo::FsType::Other(fsstring) => match fsstring.as_str() {
                "cifs" => {
                    true
                },
                _ => false
            },
            _ => false
        }
    });

    println!("Scanning for hanging mounts");
    for mount_point in relevant_mounts {
        let path = mount_point.path.to_owned();
        let fstype = match &mount_point.fstype {
            mountinfo::FsType::Other(fsstring) => fsstring,
            _ => ""
        };
        let what = &mount_point.what;
        println!("Processing {fstype} @ {path:?} from {what}");
        process_mount(fstype, what, path);
        
    }
}

fn process_mount(fstype: &str, what: &String, path: PathBuf) {
    match fstype {
        "cifs" => {
                let path_parts = what.replace("//","/");
                let host = path_parts.split("/").nth(1).unwrap(); 
                println!("Checking ping to host {host}");
                let ips = lookup_host(host).unwrap();
                let ip = ips.first().unwrap();
                println!("Using ip {ip:?}");
                match ping::ping(ip.to_owned(),Some(Duration::from_millis(100)),Some(10),None, None, None) {
                    Ok(_) => {
                        println!("Ping ok, proceeding")
                    },
                    Err(err) => {
                        println!("Ping failed with errror {err:?} -> unmount");
                        remove_mount(path.as_os_str());
                    },
                }
            },
            _ => {}
    }


    let timeout = Duration::from_millis(500);
    println!("Checking file access timeout {timeout:?}");
    let (sender, receiver) = mpsc::channel();
    let path_copy = path.clone();
    thread::spawn(move || {
        match fs::read_dir(path_copy) {
            Ok(_) => {
                match sender.send(()){
                    Ok(_) => {},
                    Err(err) => {
                        println!("Listing succeded, but probably after timeout, ignore me {err:?}")
                    },
                }
            },
            Err(err) => {
                println!("Listing failed, probably after timeout, ignore me {err:?}")
            },
        }
    });
    let read_successful = receiver.recv_timeout(timeout);
    match read_successful {
        Ok(_) => {
            println!("passed, mount still valid")
        },
        Err(err) => {
            println!("Listing failed with errror {err:?} -> unmount");
            remove_mount(path.as_os_str());
        },
    }
}

fn remove_mount(path: &OsStr){
    //unmount(path, UnmountFlags::FORCE | UnmountFlags::DETACH).unwrap()
}